#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <rsbac/types.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include <rsbac/aci_data_structures.h>
#include "nls.h"

const char   set_prog[] = "attr_set_file_dir";
enum rsbac_attribute_t attr_list[RSBAC_FD_NR_ATTRIBUTES] = RSBAC_FD_ATTR_LIST;
int recurse = 0;
int verbose = 0;
int mignore = 0;
char * filename = NULL;

struct rsbac_file_dir_aci_t default_fd_aci = DEFAULT_FD_ACI_MAC_INH;
struct rsbac_file_dir_aci_t default_fd_aci_no_mac_inh = DEFAULT_FD_ACI_NO_MAC_INH;

int process(char * name, FILE * tfile)
  {
    __s64 def_attr[RSBAC_FD_NR_ATTRIBUTES] = {
      default_fd_aci.sec_level,
      default_fd_aci.mac_trusted_for_user,
      default_fd_aci.mac_categories,
      default_fd_aci.object_category,
      default_fd_aci.data_type,
      default_fd_aci.pm_object_class,
      default_fd_aci.pm_tp,
      default_fd_aci.pm_object_type,
      default_fd_aci.ms_scanned,
      default_fd_aci.ms_trusted,
      default_fd_aci.ms_sock_trusted_tcp,
      default_fd_aci.ms_sock_trusted_udp,
      default_fd_aci.ff_flags,
      default_fd_aci.rc_type_fd,
      default_fd_aci.rc_force_role,
      default_fd_aci.rc_initial_role,
      default_fd_aci.auth_may_setuid,
      default_fd_aci.auth_may_set_cap
      };
    int res = 0;
    char tmp1[120];
    char tmp2[65];
    enum rsbac_target_t type;
    char type_n[8] = "";
    int j;
    struct stat buf;
    union rsbac_attribute_value_t value;

    if (lstat(name,&buf))
      {
        fprintf(stderr, gettext("stat for file %s returned error: %s\n"),
               name,
               strerror(errno));
        return(1);
      }
    if(S_ISDIR(buf.st_mode))
      {
        type = T_DIR;
        strcpy(type_n,"DIR");
      }
    else
    if(   (S_ISREG(buf.st_mode))
       || (S_ISBLK(buf.st_mode))
       || (S_ISCHR(buf.st_mode)) )
      {
        type = T_FILE;
        strcpy(type_n,"FILE");
      }
    else
    if(S_ISFIFO(buf.st_mode))
      {
        type = T_FIFO;
        strcpy(type_n,"FIFO");
      }
    else
    if(S_ISLNK(buf.st_mode))
      {
        type = T_SYMLINK;
        strcpy(type_n,"SYMLINK");
      }
    else
      {
        if(verbose)
          printf(gettext("%s is no FD type, skipped\n"),
                 name);
        return(-1);
      }
    if(verbose)
      printf(gettext("Processing %s '%s'\n"), type_n, name);
    for (j=0;j < RSBAC_FD_NR_ATTRIBUTES;j++)
      {
        value.dummy = -1;
        res = rsbac_get_attr_n(type, name, attr_list[j], &value, 0);
        if(res)
          {
            get_error_name(tmp1,res);
            fprintf(stderr, "%s (%s): %s\n",
                    name,
                    get_attribute_name(tmp2,attr_list[j]),
                    tmp1);
          }
        else
          switch(attr_list[j])
            {
              case A_log_array_low:
              case A_log_array_high:
                if (value.log_array_low != -1)
                  fprintf(tfile,
                          "%s %s %s %s %s\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          u64tostrlog(tmp2,value.log_array_low));
                break;
              case A_log_program_based:
                if (value.log_program_based != 0)
                  fprintf(tfile,
                          "%s %s %s %s %s\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          u64tostrlog(tmp2,value.log_program_based));
                break;
              case A_mac_trusted_for_user:
                if (value.mac_trusted_for_user != def_attr[j])
                  fprintf(tfile,
                          "%s %s %s %s %u\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.mac_trusted_for_user);
                break;
              case A_ff_flags:
                if (value.ff_flags != def_attr[j])
                  fprintf(tfile,
                          "%s %s %s %s %u\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.ff_flags);
                break;
              case A_rc_type_fd:
              case A_rc_force_role:
              case A_rc_initial_role:
                if (value.rc_type_fd != def_attr[j])
                  fprintf(tfile,
                          "%s %s %s %s %u\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.rc_type_fd);
                break;
              case A_security_level:
                if (value.security_level != default_fd_aci.sec_level)
                  fprintf(tfile,
                          "%s %s %s %s %u\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.security_level);
                break;
              case A_ms_scanned:
                if (   !mignore
                    && (value.ms_scanned != default_fd_aci.ms_scanned)
                   )
                  fprintf(tfile,
                          "%s %s %s %s %u\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.ms_scanned);
                break;
              default:
                if(value.dummy != def_attr[j])
                  fprintf(tfile,
                          "%s %s %s %s %i\n",
                          set_prog,
                          type_n,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.dummy);
            }
      }
    if (   (type == T_DIR)
        && recurse)
      {
        DIR * dir_stream_p;
        struct dirent * dirent_p;
        char name2[512];

        if(S_ISLNK(buf.st_mode))
          return(0);
        if(!(dir_stream_p = opendir(name)))
          {
            fprintf(stderr, gettext("opendir for dir %s returned error: %s\n"),
                   name,
                   strerror(errno));
            return(-2);
          }
        while((dirent_p = readdir(dir_stream_p)))
          {
            if(   (strcmp(".",dirent_p->d_name))
               && (strcmp("..",dirent_p->d_name)) )
              {
                strcpy(name2,name);
                strcat(name2,"/");
                strcat(name2,dirent_p->d_name);
                process(name2, tfile);
              }
          }
        closedir(dir_stream_p);
      }
    return(0);
  }

int main(int argc, char ** argv)
{
  int res = 0;
  char tmp1[80],tmp2[256];
  int i,j;
  FILE * tfile;
  char * progname;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  progname = argv[0];
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'r':
                recurse=1;
                break;
              case 'm':
                mignore=1;
                break;
              case 'i':
                default_fd_aci = default_fd_aci_no_mac_inh;
                break;
              case 'o':
                if(argc > 2)
                  {
                    filename = argv[2];
                    argv++;
                    argc--;
                  }
                else
                  fprintf(stderr, gettext("%s: missing filename for parameter o\n"), progname);
                break;
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if (argc > 1)
    {
      if(!filename)
        tfile = stdout;
      else
        {
          if (!(tfile=fopen(filename,"w")))
            {
              fprintf(stderr, gettext("opening target file returned error: %s\n"),
                      strerror(errno));
            }
        }
      if(verbose)
        {
          printf(gettext("%s: %i targets"), progname, argc - 2);
          if(recurse)
            printf(gettext(" - recursing"));
          printf("\n");
        }
      for (i=1;i < (argc);i++)
        {
          process(argv[i],tfile);
        }
      if(tfile != stdout)
        fclose(tfile);
    }
  else
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), argv[0], VERSION);
      printf(gettext("Use: %s [options] file/dirname(s)\n"), progname);  
      printf(gettext("- should be called by root with full attribute read access,\n- e.g. with all modules off\n"));
      printf(gettext("- -r = recurse in subdirs, -v = verbose, no symlinks followed,\n"));
      printf(gettext("- -m = ignore ms_scanned,\n"));
      printf(gettext("- -i = use MAC non-inherit values as default values,\n"));
      printf(gettext("- -o target-file = write to file, not stdout,\n"));
      printf(gettext("- attr_listibutes and values in backup = see following list:\n"));
      for (j=0;j<RSBAC_FD_NR_ATTRIBUTES;j++)
        {
          get_attribute_name(tmp1,attr_list[j]);
          get_attribute_param(tmp2,attr_list[j]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
    }
  return (res);
}
