#include <stdio.h>
#include <stdlib.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "nls.h"

int main(int argc, char ** argv)
{
  enum rsbac_attribute_t attr_list[RSBAC_FD_NR_ATTRIBUTES] = RSBAC_FD_ATTR_LIST;
  enum rsbac_attribute_t attr_list_dev[RSBAC_DEV_NR_ATTRIBUTES] = RSBAC_DEV_ATTR_LIST;
  int res = 0;
  char tmp1[80],tmp2[RSBAC_MAXNAMELEN];
  int j;
  union rsbac_attribute_value_t value,value2;
  enum rsbac_target_t target;
  enum rsbac_attribute_t attr;
  enum rsbac_adf_request_t request;
  enum rsbac_log_level_t   log_level;
  int inherit = 0;
  u_int position;
  char * progname = argv[0];
  int verbose = 0;
  int printall = 0;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);
  
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'p':
                printall=1;
                break;
              case 'e':
                inherit=1;
                break;
              case 'n':
                {
                  char tmp[80];
                  int i;

                  for(i=0; i<R_NONE; i++)
                    printf("%s\n", get_request_name(tmp, i));
                  exit(0);
                }
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  switch(argc)
    {
      case 4:
        target = get_target_nr(argv[1]);
        attr = get_attribute_nr(argv[3]);
        value.dummy = -1;
        res = rsbac_get_attr_n(target, argv[2], attr, &value, inherit);
        error_exit(res);
        switch (attr)
          {
            case A_log_array_low:
            case A_log_array_high:
              printf("%s\n",u64tostrlog(tmp1,value.log_array_low));
              break;
            case A_log_program_based:
              if(printall)
                {
                  int i;

                  for (i=0; i<R_NONE; i++)
                    if(value.log_program_based & ((rsbac_request_vector_t) 1 << i))
                      printf(" %s\n", get_request_name(tmp1,i));
                }
              else
                printf("%s\n",u64tostrlog(tmp1,value.log_array_low));
              break;
            case A_mac_categories:
              printf("%s\n",u64tostrmac(tmp1,value.mac_categories));
              break;
            case A_mac_trusted_for_user:
              printf("%u\n",value.mac_trusted_for_user);
              break;
            case A_ff_flags:
              printf("%u\n",value.u_short_dummy);
              break;
            case A_rc_type:
            case A_rc_type_fd:
            case A_rc_force_role:
            case A_rc_initial_role:
            case A_rc_role:
            case A_rc_def_role:
            case A_security_level:
              printf("%u\n",value.u_char_dummy);
              break;
            default:
              printf("%i\n",value.dummy);
          }
        exit(0);

      case 5:
        if(!strcmp(argv[3],"log_level"))
          {
            target = get_target_nr(argv[1]);
            request = get_request_nr(argv[4]);
            if(request == R_NONE)
              {
                fprintf(stderr, gettext("Invalid request type %s\n"), argv[4]);
                printf(gettext("Valid request types:\n"));
                for(j=0;j<R_NONE;j++)
                  {
                    printf("%s\n", get_request_name(tmp1,j));
                  }
                exit(1);
              }
            res = rsbac_get_attr_n(target, argv[2], A_log_array_low, &value, inherit);
            error_exit(res);
            res = rsbac_get_attr_n(target, argv[2], A_log_array_high, &value2, inherit);
            error_exit(res);
            log_level =     ((value.log_array_low   >> request) & 1)
                        | ( ((value2.log_array_high >> request) & 1) << 1);
            printf("%i\n",log_level);
            exit(0);
          }
        else
        if(!strcmp(argv[3],"mac_categories"))
          {
            target = get_target_nr(argv[1]);
            position = strtol(argv[4],0,10);
            if(position > RSBAC_MAC_MAX_CAT)
              {
                fprintf(stderr, gettext("Invalid position counter %s\n"), argv[4]);
                exit(1);
              }
            res = rsbac_get_attr_n(target, argv[2], A_mac_categories, &value, inherit);
            error_exit(res);
            printf("%u\n",
                   (u_int) (value.mac_categories >> position) & 1);
            exit(0);
          }
        else
        if(!strcmp(argv[3],"log_program_based"))
          {
            target = get_target_nr(argv[1]);
            request = get_request_nr(argv[4]);
            if(request == R_NONE)
              {
                fprintf(stderr, gettext("Invalid request type %s\n"), argv[4]);
                printf(gettext("Valid request types:\n"));
                for(j=0;j<R_NONE;j++)
                  {
                    printf("%s\n", get_request_name(tmp1,j));
                  }
                exit(1);
              }
            res = rsbac_get_attr_n(target, argv[2], A_log_program_based, &value, inherit);
            error_exit(res);
            printf("%u\n",
                   (u_int) (value.log_program_based >> request) & 1);
            exit(0);
          }

      case 2:
        if(!strcmp(argv[1],"list_category_nr"))
          {
            for(j=0; j<RSBAC_MAC_NR_CATS; j++)
              printf("%i\n", j);
            exit(0);
          }

      default:
        printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
        printf(gettext("Use: %s target-type file/dirname attribute [request]\n"), progname);  
        printf(gettext("Use: %s target-type file/dirname attribute [position]\n"), progname);  
        printf(gettext("Use: %s list_category_nr\n"), progname);  
        printf(gettext("- -e = show effective (maybe inherited) value, not real\n"));
        printf(gettext("- -p = print requests, -n list all requests\n"));
        printf(gettext("- target-type = FILE, DIR, FIFO, SYMLINK or DEV,\n"));
        printf(gettext("- attribute (string) and returned value = see following list:\n"));
        printf(gettext("- FILE, DIR, FIFO and SYMLINK:\n"));
        printf(gettext("log_level\t\t(additional parameter request-type)\n\t\t\t0=none, 1=denied, 2=full, 3=request based\n"));
        printf(gettext("mac_categories\t\t(with additional parameter position)\n\t\t\t0=no, 1=yes\n"));
        for (j=0;j<RSBAC_FD_NR_ATTRIBUTES;j++)
          {
            get_attribute_name(tmp1,attr_list[j]);
            get_attribute_param(tmp2,attr_list[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
        printf("- DEV:\n");
        printf(gettext("log_level\t\t(additional parameter request-type)\n\t\t\t0=none, 1=denied, 2=full, 3=request based\n"));
        printf(gettext("mac_categories\t\t(with additional parameter position)\n\t\t\t0=no, 1=yes\n"));
        for (j=0;j<RSBAC_DEV_NR_ATTRIBUTES;j++)
          {
            get_attribute_name(tmp1,attr_list_dev[j]);
            get_attribute_param(tmp2,attr_list_dev[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
      }
  exit(1);
}
