#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>
#include <rsbac/types.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/aci_data_structures.h>
#include "nls.h"

int main(int argc, char ** argv)
{
  int res = 0;
  char tmp1[RSBAC_MAXNAMELEN],tmp2[RSBAC_MAXNAMELEN];
  char *key = (void *) 0;
  int i;
  int id;
  union rsbac_attribute_value_t value;
  enum rsbac_target_t target;
  union rsbac_target_id_t tid;
  enum rsbac_attribute_t attr;
  struct passwd * user_info_p;
  char * progname = argv[0];
  int inherit = 0;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);
  
  if ((argc > 1) && (!strcmp("-e",argv[1])) )
    {
      inherit=1;
      argv++;
      argc--;
    }  
  
  if (argc > 3)
    {
      printf(gettext("%s: %i targets\n\n"), progname, argc - 3);
      target = get_target_nr(argv[1]);
      if(   (target != T_PROCESS)
         && (target != T_USER))
        {
          fprintf(stderr, gettext("%s: Invalid Target %s!\n"), progname, argv[1]);
          exit(1);
        }
      attr = get_attribute_nr(argv[2]);
      if(attr == A_none)
        {
          fprintf(stderr, gettext("%s: Invalid Attribute %s!\n"), progname, argv[2]);
          exit(1);
        }
      for (i=1;i < (argc-2);i++)
        {
          value.dummy = -1;
          if (target == T_PROCESS)
            {
              id = strtol(argv[i+2],0,10);
              printf(gettext("Processing process %i, attribute %s (No. %i)\n"),
                      id, argv[2], attr);
              tid.process = id;
              inherit = 0;
            }
          else
            {
              if(   (!(user_info_p = getpwnam(argv[i+2])))
                 && (!(user_info_p = getpwuid(strtol(argv[i+2],0,10)))) )
                {
                  fprintf(stderr, gettext("Invalid user %s!\n\n"), argv[i+2]);
                  continue;
                }
              printf(gettext("Processing user %s (uid %i), attribute %s (No. %i)\n"),
                     user_info_p->pw_name, user_info_p->pw_uid, argv[2], attr);
              tid.user = user_info_p->pw_uid;
            }

          res = rsbac_get_attr(target, &tid, attr, &value, inherit);
          error_exit(res);
          switch(attr)
            {
              case A_rc_type:
              case A_rc_type_fd:
              case A_rc_force_role:
              case A_rc_role:
              case A_rc_def_role:
              case A_security_level:
              case A_current_sec_level:
              case A_min_write_open:
              case A_max_read_open:
                printf(gettext("Returned value: %u\n"),value.u_char_dummy);
                break;
              default:
                printf(gettext("Returned value: %i\n"),value.dummy);
            }
        }
    }
  else
    {
      int u_attr_list[RSBAC_USER_NR_ATTRIBUTES] = RSBAC_USER_ATTR_LIST;
      int p_attr_list[RSBAC_PROCESS_NR_ATTRIBUTES] = RSBAC_PROCESS_ATTR_LIST;

      printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
      printf(gettext("Use: %s target-type attribute user(s)/proc-no.\n\n"), progname);  
      printf(gettext("- target-type = USER or PROCESS,\n"));
      printf(gettext("- attribute (string) and returned value = see following list:\n"));
      printf("USER:\n");
      for (i=0;i<RSBAC_USER_NR_ATTRIBUTES;i++)
        {
          get_attribute_name(tmp1,u_attr_list[i]);
          get_attribute_param(tmp2,u_attr_list[i]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
      printf("\nPROCESS:\n");
      for (i=0;i<RSBAC_PROCESS_NR_ATTRIBUTES;i++)
        {
          get_attribute_name(tmp1,p_attr_list[i]);
          get_attribute_param(tmp2,p_attr_list[i]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
    }
    
  return (res);
}
