#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <rsbac/types.h>
#include <rsbac/syscalls.h>
#include <rsbac/getname.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "nls.h"

int main(int argc, char ** argv)
{
  int res = 0;
  rsbac_pid_t pid=0;
  rsbac_uid_t first_uid;
  rsbac_uid_t last_uid;
  enum rsbac_target_t target;
  char * progname;
  struct passwd * user_info_p;
  boolean remove = FALSE;
  int maxnum = 20;
  rsbac_uid_t caplist[40];
  int i;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);
  
  progname = argv[0];
  switch(argc)
    {
      case 5:
      case 6:
        target = get_target_nr(argv[1]);
        if(   (target != T_PROCESS)
           && (target != T_FILE))
          {
            fprintf(stderr, gettext("%s: Invalid Target %s!\n"), progname, argv[1]);
            exit(1);
          }
        if(!strcmp(argv[2],"remove"))
          remove = TRUE;
        else
          if(strcmp(argv[2],"add"))
            {
              fprintf(stderr, gettext("%s: Invalid command %s!\n\n"), argv[0], argv[2]);
              exit(1);
            }
        if(target == T_PROCESS)
          pid=strtol(argv[3],0,10);
        if(   (!(user_info_p = getpwnam(argv[4])))
           && (   (!(user_info_p = getpwuid(strtoul(argv[4],0,10))))
               || (!(user_info_p->pw_uid) && strcmp("0",argv[4])) )
          )
          {
/*            fprintf(stderr, gettext("%s: Warning: Unknown user %s, trying number!\n\n"), argv[0], argv[4]); */
            first_uid = strtoul(argv[4],0,10);
            if(!first_uid)
              {
                fprintf(stderr, gettext("%s: Warning: user %s invalid, exiting!\n\n"), argv[0], argv[4]);
                exit(1);
              }
          }
        else
          {
            first_uid = user_info_p->pw_uid;
          }
        if(argc == 6)
          {
            if(   (!(user_info_p = getpwnam(argv[5])))
               && (   (!(user_info_p = getpwuid(strtoul(argv[5],0,10))))
                   || (!(user_info_p->pw_uid) && strcmp("0",argv[5])) )
              )
              {
                last_uid = strtoul(argv[5],0,10);
                if(!last_uid)
                  {
                    fprintf(stderr, gettext("%s: Warning: user %s invalid, exiting!\n\n"),
                            argv[0], argv[5]);
                    exit(1);
                  }
                if(last_uid < first_uid)
                  {
                    fprintf(stderr, gettext("%s: Warning: first user %u after last user %u, exiting!\n\n"),
                            argv[0], first_uid, last_uid);
                    exit(2);
                  }
              }
            else
              {
                last_uid = user_info_p->pw_uid;
              }
          }
        else
          { /* no last_user given */
            last_uid = first_uid;
          }
        if(target == T_PROCESS)
          {
            if(remove)
              res = rsbac_auth_remove_p_cap(pid, first_uid, last_uid);
            else
              res = rsbac_auth_add_p_cap(pid, first_uid, last_uid);
          }
        else
          {
            if(remove)
              res = rsbac_auth_remove_f_cap(argv[3], first_uid, last_uid);
            else
              res = rsbac_auth_add_f_cap(argv[3], first_uid, last_uid);
          }
        if(res < 0)
          error_exit(res);
        break;

      case 4:
        target = get_target_nr(argv[1]);
        if(target != T_FILE)
          {
            fprintf(stderr, gettext("%s: Invalid Target %s!\n"), progname, argv[1]);
            exit(1);
          }
        if(strcmp(argv[2],"get"))
          {
            fprintf(stderr, gettext("%s: Invalid command %s!\n\n"), argv[0], argv[2]);
            exit(1);
          }
        res = rsbac_auth_get_f_caplist(argv[3], caplist, maxnum);
        if(res < 0)
          error_exit(res);
        for(i=0; i<res; i++)
          if(caplist[2*i] != caplist[2*i+1])
            printf("%u:%u ", caplist[2*i], caplist[2*i+1]);
          else
            printf("%u ", caplist[2*i]);
        printf("\n");
        break;

      default:
        printf(gettext("%s (RSBAC %s)\n***\n"), argv[0], VERSION);
        printf(gettext("Use: %s TYPE add/remove target first_user [last_user]\n"), progname);
        printf(gettext("Use: %s FILE get filename\n"), progname);
        printf(gettext("     TYPE = PROCESS or FILE,\n"));
        printf(gettext("     target = pid or filename\n"));
        printf(gettext("     last_user: range from first_user to last_user\n"));
    }
  exit(res);
}
