#include <stdio.h>
#include <stdlib.h>
#include <rsbac/types.h>
#include <rsbac/rc_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/rc_getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include "nls.h"

int main(int argc, char ** argv)
{
  int res = 0;
  int comp_type;
  enum rsbac_adf_request_t right;
  rsbac_rc_rights_vector_t rights_vector = 0;
  char tmp1[80],tmp2[120];
  int i,j;
  boolean rused = FALSE;
  boolean wused = FALSE;
  enum rsbac_rc_target_t target;
  union rsbac_rc_target_id_t tid;
  enum rsbac_rc_item_t item;
  union rsbac_rc_item_value_t value;
  char * progname = argv[0];
  int role_entry_item_list[RSBAC_RC_NR_ROLE_ENTRY_ITEMS] = RSBAC_RC_ROLE_ENTRY_ITEM_LIST;
  int type_entry_item_list[RSBAC_RC_NR_TYPE_ENTRY_ITEMS] = RSBAC_RC_TYPE_ENTRY_ITEM_LIST;
  int verbose=0;
  int printall=0;
  int add=0;
  int revoke=0;
  int bitstring=0;
  int copy=0;
  int delrights=0;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'p':
                printall=1;
                break;
              case 'a':
                add=1;
                break;
              case 'k':
                revoke=1;
                break;
              case 'b':
                bitstring=1;
                break;
              case 'c':
                copy=1;
                break;
              case 'd':
                delrights=1;
                break;
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if(argc < 4)
    {
        printf(gettext("%s (RSBAC %s)\n***\n"), progname, VERSION);
        printf(gettext("Use: %s [switches] rc-target-type id item [role/type [list-of-rights]] [value]\n"), progname);  
        printf(gettext("     %s -c TYPE target-id item source-id [first_role [last_role]],\n"), progname);
        printf(gettext("  -v = verbose, -p = print right names,\n"));
        printf(gettext("  -a = add, not set, -k = revoke, not set,\n"));
        printf(gettext("  -b = accept rights as bitstring,\n"));
        printf(gettext("  -c = copy all/given roles' rights to type from other type,\n"));
        printf(gettext("  -d = delete all roles' rights to this type,\n"));
        printf(gettext("- rc-target-type = ROLE or TYPE,\n"));
        printf(gettext("- id-nr = ROLE or TYPE number,\n"));
        printf(gettext("- item = entry line,\n"));
        printf(gettext("- role/type = for this type only (required for type_comp items only),\n"));
        printf(gettext("- right = request name or number (type_comp items only),\n"));
        printf(gettext("        also special rights and groups R (read requests),\n"));
        printf(gettext("        RW (read-write), SY (system), SE (security), A (all)\n"));
        printf(gettext("- item (string) and value = see following list:\n"));
        printf("- ROLE:\n");
        for (j=0;j<RSBAC_RC_NR_ROLE_ENTRY_ITEMS;j++)
          {
            get_rc_item_name(tmp1,role_entry_item_list[j]);
            get_rc_item_param(tmp2,role_entry_item_list[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
        printf("- TYPE:\n");
        for (j=0;j<RSBAC_RC_NR_TYPE_ENTRY_ITEMS;j++)
          {
            get_rc_item_name(tmp1,type_entry_item_list[j]);
            get_rc_item_param(tmp2,type_entry_item_list[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
        exit(1);
    }  

  target = get_rc_target_nr(argv[1]);
  switch(target)
    {
      case RT_ROLE:
        tid.role = strtol(argv[2],0,10);
        break;
      case RT_TYPE:
        tid.type = strtol(argv[2],0,10);
        break;
      default:
        fprintf(stderr, gettext("Invalid target %s\n"), argv[1]);
        exit(1);
    }
  item = get_rc_item_nr(argv[3]);
  if(item == RI_none)
    {
      fprintf(stderr, gettext("Invalid item %s\n"), argv[3]);
      exit(1);
    }

  if(copy)
    {
      rsbac_rc_type_id_t s_type;
      rsbac_rc_type_id_t t_type;
      rsbac_rc_role_id_t first_role = 0;
      rsbac_rc_role_id_t last_role = RSBAC_RC_MAX_ROLE;
      int tmpres = 0;

      if(argc < 5)
        {
          fprintf(stderr, gettext("Too few arguments with option -c\n"));
          exit(1);
        }
      s_type = strtol(argv[4],0,10);
      if(s_type > RSBAC_RC_MAX_TYPE)
        {
          fprintf(stderr, gettext("Invalid source type %u\n"), s_type);
          exit(1);
        }
      if(argc > 5)
        {
          first_role = strtol(argv[5],0,10);
          if(first_role > RSBAC_RC_MAX_ROLE)
            {
              fprintf(stderr, gettext("Invalid first role %u\n"), first_role);
              exit(1);
            }
          last_role = first_role;
        }
      if(argc > 6)
        {
          last_role = strtol(argv[6],0,10);
          if(last_role > RSBAC_RC_MAX_ROLE)
            {
              fprintf(stderr, gettext("Invalid last role %u\n"), last_role);
              exit(1);
            }
        }
      t_type = tid.type;
      if(t_type > RSBAC_RC_MAX_TYPE)
        {
          fprintf(stderr, gettext("Invalid target type %u\n"), t_type);
          exit(1);
        }
      if(s_type == t_type)
        {
          fprintf(stderr, gettext("Source and target must differ\n"));
          exit(1);
        }
      switch(item)
        {
          case RI_type_comp_fd:
          case RI_type_fd_name:
            item = RI_type_comp_fd;
            break;
          case RI_type_comp_dev:
          case RI_type_dev_name:
            item = RI_type_comp_dev;
            break;
          case RI_type_comp_process:
          case RI_type_process_name:
            item = RI_type_comp_process;
            break;
          case RI_type_comp_ipc:
          case RI_type_ipc_name:
            item = RI_type_comp_ipc;
            break;
          default:
            fprintf(stderr, gettext("Invalid item %s\n"), argv[3]);
            exit(1);
        }
      if(verbose)
        {
          printf(gettext("Copying rights vector %s for type %u to type %u in role(s) %u to %u\n"),
                 get_rc_item_name(tmp1, item),
                 s_type,
                 t_type,
                 first_role,
                 last_role);
        }
      for(i=first_role; i <= last_role; i++)
        {
          tid.role = i;
          if(!(tmpres = rsbac_rc_get_item(RT_ROLE, &tid, item, &value)))
            {
              value.type_comp[t_type] = value.type_comp[s_type];
              if((tmpres = rsbac_rc_set_item(RT_ROLE, &tid, item, &value)))
                {
                  fprintf(stderr, gettext("Changing role %u failed: %s\n"),
                          i,
                          get_error_name(tmp1, tmpres));
                  res = tmpres;
                }
            }
          else
            {
              if(tmpres != -RSBAC_EINVALIDTARGET)
                {
                  fprintf(stderr, gettext("Reading from role %u failed: %s\n"),
                          i,
                          get_error_name(tmp1, tmpres));
                  res = tmpres;
                }
            }
        }
      exit(res);
    } /* end of type rights copy */

  if(delrights)
    {
      int tmpres = 0;
      rsbac_rc_type_id_t t_type;

      t_type = tid.type;
      if(t_type > RSBAC_RC_MAX_TYPE)
        {
          fprintf(stderr, gettext("Invalid target type %u\n"), t_type);
          exit(1);
        }
      switch(item)
        {
          case RI_type_comp_fd:
          case RI_type_fd_name:
            item = RI_type_comp_fd;
            break;
          case RI_type_comp_dev:
          case RI_type_dev_name:
            item = RI_type_comp_dev;
            break;
          case RI_type_comp_process:
          case RI_type_process_name:
            item = RI_type_comp_process;
            break;
          case RI_type_comp_ipc:
          case RI_type_ipc_name:
            item = RI_type_comp_ipc;
            break;
          default:
            fprintf(stderr, gettext("Invalid item %s\n"), argv[3]);
            exit(1);
        }
      if(verbose)
        {
          printf(gettext("Setting rights vector %s for type %u in all roles to 0\n"),
                 get_rc_item_name(tmp1, item),
                 t_type);
        }
      for(i=0; i < RSBAC_RC_NR_ROLES; i++)
        {
          tid.role = i;
          if(!(tmpres = rsbac_rc_get_item(RT_ROLE, &tid, item, &value)))
            {
              value.type_comp[t_type] = (rsbac_rc_type_vector_t) 0;
              if((tmpres = rsbac_rc_set_item(RT_ROLE, &tid, item, &value)))
                {
                  fprintf(stderr, gettext("Changing role %u failed: %s\n"),
                          i,
                          get_error_name(tmp1, tmpres));
                  res = tmpres;
                }
            }
          else
            {
              if(tmpres != -RSBAC_EINVALIDTARGET)
                {
                  fprintf(stderr, gettext("Reading from role %u failed: %s\n"),
                          i,
                          get_error_name(tmp1, tmpres));
                  res = tmpres;
                }
            }
        }
      exit(res);
    } /* end of type rights copy */

  switch (item)
    {
      case RI_role_comp:
      case RI_admin_roles:
      case RI_assign_roles:
        {
          rsbac_rc_role_vector_t rvec=0;

          if(add || revoke)
            {
              res = rsbac_rc_get_item(target, &tid, item, &value);
              error_exit(res);
            }
          else
            value.role_comp = 0;

          for(i=4;i<argc;i++)
            {
              if(strlen(argv[i]) == RSBAC_RC_NR_ROLES)
                {
                  rsbac_rc_role_vector_t tmp_rvec=0;

                  strtou64rc(argv[i],&tmp_rvec);
                  rvec |= tmp_rvec;
                }
              else
                {
                  rvec |= RSBAC_RC_ROLE_VECTOR(strtol(argv[i],0,10));
                }
            }
          if(add)
            {
              if(verbose)
                {
                  printf(gettext("Adding to %s of ROLE %i\n"),
                         get_rc_item_name(tmp1, item), tid.role);
                }
              value.role_comp |= rvec;
            }
          else
          if(revoke)
            {
              if(verbose)
                {
                  printf(gettext("Revoking from %s of ROLE %i\n"),
                         get_rc_item_name(tmp1, item), tid.role);
                }
              value.role_comp &= ~rvec;
            }
          else
            {
              if(verbose)
                {
                  printf(gettext("Setting %s of ROLE %i\n"),
                         get_rc_item_name(tmp1, item), tid.role);
                }
              value.role_comp = rvec;
            }
          if(printall)
            {
              for (i=0; i<RSBAC_RC_NR_ROLES; i++)
                if(rvec & RSBAC_RC_ROLE_VECTOR(i))
                  printf("  %i\n", i);
            }
        }
        break;

      case RI_type_comp_fd:
      case RI_type_comp_dev:
      case RI_type_comp_process:
      case RI_type_comp_ipc:
      case RI_type_comp_scd:
        if(argc<5)
          {
            fprintf(stderr, gettext("parameter comp_type missing\n"));
            exit(1);
          }
        comp_type = strtol(argv[4],0,10);
        if(comp_type > RSBAC_RC_MAX_TYPE)
          {
            fprintf(stderr, gettext("invalid comp_type %s\n"), argv[4]);
            exit(1);
          }
        res = rsbac_rc_get_item(target, &tid, item, &value);
        error_exit(res);
        if(bitstring && (argc > 5))
          {
            if(strlen(argv[5]) != RCR_NONE)
              {
                fprintf(stderr, gettext("Invalid bitstring length %u, must be %u!\n"),
                        strlen(argv[5]),
                        RCR_NONE);
                exit(1);
              }
            strtou64rcr(argv[5], &rights_vector);
            argv++;
            argc--;
          }
        else
        for(i=5; i<argc; i++)
          {
            right = get_rc_special_right_nr(argv[i]);
            if((right == R_NONE) || (right == RCR_NONE))
              {
                right = strtol(argv[i],0,10);
                if(   (   (right >= R_NONE)
                       && (right < RSBAC_RC_SPECIAL_RIGHT_BASE)
                      )
                   || (right >= RCR_NONE)
                   || (   (right == 0)
                       && strcmp(argv[i],"0")
                      )
                  )
                  {
                    if(!strcmp(argv[i],"W"))
                      {
                        rights_vector |= RSBAC_WRITE_REQUEST_VECTOR;
                        wused = TRUE;
                      }
                    else
                    if(!strcmp(argv[i],"RW"))
                      {
                        rights_vector |= RSBAC_READ_WRITE_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"SY"))
                      {
                        rights_vector |= RSBAC_SYSTEM_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"SE"))
                      {
                        rights_vector |= RSBAC_SECURITY_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"S"))
                      {
                        rights_vector |= RSBAC_RC_SPECIAL_RIGHTS_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"R"))
                      {
                        rights_vector |= RSBAC_READ_REQUEST_VECTOR;
                        rused = TRUE;
                      }
                    else
                    if(!strcmp(argv[i],"UA"))
                      {
                        rights_vector &= RSBAC_RC_SPECIAL_RIGHTS_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"A"))
                      {
                        rights_vector |= RSBAC_ALL_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"NWS"))
                      {
                        rights_vector |= RSBAC_NWS_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"NWR"))
                      {
                        rights_vector |= RSBAC_NWR_REQUEST_VECTOR;
                        rused = TRUE;
                      }
                    else
                    if(!strcmp(argv[i],"NWW"))
                      {
                        rights_vector |= RSBAC_NWW_REQUEST_VECTOR;
                        wused = TRUE;
                      }
                    else
                    if(!strcmp(argv[i],"NWC"))
                      {
                        rights_vector |= RSBAC_NWC_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"NWE"))
                      {
                        rights_vector |= RSBAC_NWE_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"NWA"))
                      {
                        rights_vector |= RSBAC_NWA_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"NWF"))
                      {
                        rights_vector |= RSBAC_NWF_REQUEST_VECTOR;
                      }
                    else
                    if(!strcmp(argv[i],"NWM"))
                      {
                        rights_vector |= RSBAC_NWM_REQUEST_VECTOR;
                      }
                    else
                      { /* end of rights */
                        break;
                      }
                  }
              }
            else
              {
                rights_vector |= ((rsbac_rc_rights_vector_t) 1 << right);
              }
          }
        switch(item)
          {
            case RI_none:
              fprintf(stderr, gettext("Invalid item %s\n"), argv[3]);
              exit(1);
            case RI_type_comp_fd:
              rights_vector &= (RSBAC_FD_REQUEST_VECTOR | RSBAC_RC_SPECIAL_RIGHTS_VECTOR);
              break;
            case RI_type_comp_dev:
              rights_vector &= (RSBAC_DEV_REQUEST_VECTOR | RSBAC_RC_SPECIAL_RIGHTS_VECTOR);
              break;
            case RI_type_comp_process:
              rights_vector &= (RSBAC_PROCESS_REQUEST_VECTOR | RSBAC_RC_SPECIAL_RIGHTS_VECTOR);
              break;
            case RI_type_comp_ipc:
              rights_vector &= (RSBAC_IPC_REQUEST_VECTOR | RSBAC_RC_SPECIAL_RIGHTS_VECTOR);
              break;
            case RI_type_comp_scd:
              rights_vector &= (RSBAC_SCD_REQUEST_VECTOR | RSBAC_NONE_REQUEST_VECTOR | RSBAC_RC_SPECIAL_RIGHTS_VECTOR);
              break;
            default:
              rights_vector &= RSBAC_RC_ALL_RIGHTS_VECTOR;
          }
        if(add)
          {
            if(verbose)
              {
                printf(gettext("Adding %s rights for ROLE %i to TYPE %i\n"),
                       get_rc_item_name(tmp1, item), tid.role, comp_type);
              }
            value.type_comp[comp_type] |= rights_vector;
          }
        else
        if(revoke)
          {
            if(verbose)
              {
                printf(gettext("Revoking %s rights for ROLE %i from TYPE %i\n"),
                       get_rc_item_name(tmp1, item), tid.role, comp_type);
              }
            value.type_comp[comp_type] &= ~rights_vector;
          }
        else
          {
            if(verbose)
              {
                printf(gettext("Setting %s rights for ROLE %i to TYPE %i\n"),
                       get_rc_item_name(tmp1, item), tid.role, comp_type);
              }
            value.type_comp[comp_type] = rights_vector;
          }
        if(printall)
          {
            for (i=0; i<R_NONE; i++)
              if(rights_vector & RSBAC_RC_RIGHTS_VECTOR(i))
                printf("  %s\n", get_request_name(tmp1,i));
            for (i=RSBAC_RC_SPECIAL_RIGHT_BASE; i<RCR_NONE; i++)
              if(rights_vector & RSBAC_RC_RIGHTS_VECTOR(i))
                printf("  %s\n", get_rc_special_right_name(tmp1,i));
          }
        break;

      case RI_name:
      case RI_type_fd_name:
      case RI_type_dev_name:
      case RI_type_ipc_name:
      case RI_type_process_name:
        if(argc<5)
          {
            fprintf(stderr, gettext("parameter name missing\n"));
            exit(1);
          }
        if(strlen(argv[4]) >= RSBAC_RC_NAME_LEN)
          {
            fprintf(stderr, gettext("Name string too long\n"));
            exit(1);
          }
        strcpy(value.name, argv[4]);
        break;

      case RI_admin_type:
        if(argc<5)
          {
            fprintf(stderr, gettext("parameter admin_type missing\n"));
            exit(1);
          }
        value.admin_type = get_rc_admin_nr(argv[4]);
        if(value.admin_type == RC_none)
          value.admin_type = strtol(argv[4],0,10);
        break;

      default:
        value.dummy = strtol(argv[4],0,10);
    }
  res = rsbac_rc_set_item(target, &tid, item, &value);
  error_exit(res);
  exit(0);
}
